#!/bin/sh

test_header() {
	if echo "#include <$1>" | \
		$CC $CFLAGS -E - > /dev/null 2>&1
	then echo 1; else echo 0; fi
}

case "$1" in
	pcre)
		exec 2> /dev/null
		if pcre-config --version > /dev/null
		then echo 1; else echo 0; fi
		;;
	expat)
		test_header "expat.h"
		;;
	sqlite)
		test_header "sqlite3.h"
		;;
	mysql)
		test_header "mysql/mysql.h"
		;;
	openssl)
		test_header "openssl/sha.h"
		;;
	postgres)
		exec 2> /dev/null
		if pg_config --includedir > /dev/null
		then echo 1; else echo 0; fi
		;;
	libxml2)
		exec 2> /dev/null
		if xml2-config --cflags > /dev/null
		then echo 1; else echo 0; fi
		;;
	libxslt)
		exec 2> /dev/null
		if xslt-config --cflags > /dev/null
		then echo 1; else echo 0; fi
		;;
	libsdl)
		exec 2> /dev/null
		if sdl-config --cflags > /dev/null
		then
			CFLAGS="$CFLAGS $(sdl-config --cflags)"
			test_header "SDL/SDL_image.h"
		else echo 0; fi
		;;
	libuuid)
		test_header "uuid/uuid.h"
		;;
	libsmoke)
		test_header "smoke.h"
		;;
	libsmoke-config)
		tmpfile=`mktemp`
		echo "#include <smoke.h>" | $CC $CFLAGS -E - > $tmpfile
		grep -q mf_ctor $tmpfile || echo "-DSMOKE_WITHOUT_MF_CTOR"
		rm -f $tmpfile
		;;
	readline)
		test_header "readline/readline.h"
		;;
	pthread)
		exec 2> /dev/null
		if $CC -pthread -o syscheck.tmp -x c - > /dev/null <<- EOT
			#include <pthread.h>
			int main() {
			  pthread_t foobar;
			  pthread_create(&foobar, 0, 0, 0);
			  return 0;
			}
		EOT
		then echo 1; else echo 0; fi
		rm -f syscheck.tmp
		;;
	extra-cflags)
		for dir in \
			/usr/include/{kde,qt3}				\
			/opt/{kde*,qt3*,mysql}/include			\
			/{usr,opt}/local/include
		do
			[ -d $dir ] && echo "-I$dir"
		done
		;;
	extra-ldflags)
		for dir in \
			/opt/{kde*,qt*,mysql}/lib			\
			/{usr,opt}/local/lib
		do
			[ -d $dir ] && echo "-L$dir"
		done
		;;
	*)
		echo "Usage: $0 {mode}" >&2
		exit 1
esac

exit 0

