/*
 *  SPL - The SPL Programming Language
 *  Copyright (C) 2004, 2005  Clifford Wolf <clifford@clifford.at>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  webspl_common.c: Common functions for webspl.c and webspld
 */

#define _GNU_SOURCE

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <assert.h>
#include <unistd.h>
#include <stdarg.h>
#include <errno.h>
#include <dirent.h>
#include <utime.h>

#include "spl.h"
#include "webspl_common.h"
#include "compat.h"

void create_dumpdir(const char *basedir)
{
	char *dumpdir;

	my_asprintf(&dumpdir, "%s/webspl_cache", basedir ? basedir : ".");
#ifndef USEWIN32API
	mkdir(dumpdir, 0700);
#else
	mkdir(dumpdir);
#endif
	free(dumpdir);
}

void expire_dumpdir(const char *basedir, int timeout, int interval)
{
	char *dumpdir;
	char *timestamp_file;
	int dumpdir_len;
	struct stat statbuf;
	time_t now = time(0);
	DIR *dir = 0;

	my_asprintf(&dumpdir, "%s/webspl_cache", basedir ? basedir : ".");
	my_asprintf(&timestamp_file, "%s/LAST_EXPIRE", dumpdir);
	dumpdir_len = strlen(dumpdir);

#ifdef USEWIN32API
	if (stat(timestamp_file, &statbuf) || (statbuf.st_mtime+interval < now))
#else
	if (lstat(timestamp_file, &statbuf) || (statbuf.st_mtime+interval < now))
#endif
		dir = opendir(dumpdir);

	if (dir) {
		struct dirent *ent;
		while ((ent=readdir(dir)) != NULL) {
			int filename_len = dumpdir_len + strlen(ent->d_name) + 2;
			char filename[filename_len];

			snprintf(filename, filename_len, "%s/%s", dumpdir, ent->d_name);

#ifdef USEWIN32API
			if (!stat(filename, &statbuf)) {
#else
			if (!lstat(filename, &statbuf)) {
#endif
				if (statbuf.st_mtime+timeout < now)
					unlink(filename);
			}
		}
		closedir(dir);

#ifdef USEWIN32API
		close(open(timestamp_file, O_WRONLY|O_CREAT, 0666));
#else
		close(open(timestamp_file, O_WRONLY|O_NONBLOCK|O_CREAT|O_NOCTTY, 0666));
#endif
		utime(timestamp_file, NULL);
	}

	free(timestamp_file);
	free(dumpdir);
}

char *get_new_session()
{
	static char sid_chars[]="0123456789abcdefghijklmnopqrstuvwxyz"
	                                  "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	char ch, *ret = strdup("XXXXXXXXXXXXXXXXXXXXXXXX");

	int fd = open("/dev/urandom", O_RDONLY|MY_O_BINARY);
	for (int i=0; fd>0 && i<20; i++) {
		if ( read(fd, &ch,1) != 1 ) continue;
		ret[i] = sid_chars[ch % (sizeof(sid_chars)-1)];
	}
	close(fd);

	long randat = (getpid()<<16) ^ (getpid()<<24) ^ ~time(0);
	for (int i=20; i<24; i++) {
		ret[i] = sid_chars[ randat % (sizeof(sid_chars)-1) ];
		randat = randat >> 8;
	}

	return ret;
}

